/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.text.RewriteSessionEditProcessor;

import dwt.dwthelper.utils;

import dwtx.text.edits.CopyTargetEdit;
import dwtx.text.edits.DeleteEdit;
import dwtx.text.edits.InsertEdit;
import dwtx.text.edits.MalformedTreeException;
import dwtx.text.edits.MoveTargetEdit;
import dwtx.text.edits.ReplaceEdit;
import dwtx.text.edits.TextEdit;
import dwtx.text.edits.TextEditProcessor;
import dwtx.text.edits.TextEditVisitor;
import dwtx.text.edits.UndoEdit;

/**
 * A text edit processor that brackets the application of edits into a document rewrite session.
 * 
 * @since 3.3
 */
public final class RewriteSessionEditProcessor : TextEditProcessor {
    /** The threshold for <em>large</em> text edits. */
    private static final int THRESHOLD= 1000;

    /**
     * Text edit visitor that estimates the compound size of an edit tree in characters.
     */
    private static final class SizeVisitor : TextEditVisitor {
        int fSize= 0;

        public bool visit(CopyTargetEdit edit) {
            fSize += edit.getLength();
            return super.visit(edit);
        }

        public bool visit(DeleteEdit edit) {
            fSize += edit.getLength();
            return super.visit(edit);
        }

        public bool visit(InsertEdit edit) {
            fSize += edit.getText().length();
            return super.visit(edit);
        }

        public bool visit(MoveTargetEdit edit) {
            fSize += edit.getLength();
            return super.visit(edit);
        }

        public bool visit(ReplaceEdit edit) {
            fSize += Math.max(edit.getLength(), edit.getText().length());
            return super.visit(edit);
        }
    }

    /**
     * Constructs a new edit processor for the given document.
     * 
     * @param document the document to manipulate
     * @param root the root of the text edit tree describing the modifications. By passing a text
     *        edit a a text edit processor the ownership of the edit is transfered to the text edit
     *        processors. Clients must not modify the edit (e.g adding new children) any longer.
     * @param style {@link TextEdit#NONE}, {@link TextEdit#CREATE_UNDO} or
     *        {@link TextEdit#UPDATE_REGIONS})
     */
    public RewriteSessionEditProcessor(IDocument document, TextEdit root, int style) {
        super(document, root, style);
    }
    
    /*
     * @see dwtx.text.edits.TextEditProcessor#performEdits()
     */
    public UndoEdit performEdits() throws MalformedTreeException, BadLocationException {
        IDocument document= getDocument();
        if (!(document instanceof IDocumentExtension4))
            return super.performEdits();

        IDocumentExtension4 extension= (IDocumentExtension4) document;
        bool isLargeEdit= isLargeEdit(getRoot());
        DocumentRewriteSessionType type= isLargeEdit ? DocumentRewriteSessionType.UNRESTRICTED : DocumentRewriteSessionType.UNRESTRICTED_SMALL;

        DocumentRewriteSession session= extension.startRewriteSession(type);
        try {
            return super.performEdits();
        } finally {
            extension.stopRewriteSession(session);
        }
    }

    /**
     * Returns <code>true</code> if the passed edit is considered <em>large</em>,
     * <code>false</code> otherwise.
     * 
     * @param edit the edit to check
     * @return <code>true</code> if <code>edit</code> is considered <em>large</em>,
     *         <code>false</code> otherwise
     * @since 3.3
     */
    public static bool isLargeEdit(TextEdit edit) {
        SizeVisitor sizeVisitor= new SizeVisitor();
        edit.accept(sizeVisitor);
        return sizeVisitor.fSize > THRESHOLD;
    }

}
